using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using CrystalDecisions.Enterprise;
using CrystalDecisions.Enterprise.Desktop;
using CrystalDecisions.Enterprise.Admin;
using ZCSterling.BOE.BOECommonInfo;
using Oracle.DataAccess.Client;

namespace ZCSterling.BOE
{
  public class BOERptArchiver:IDisposable
  {
    private BOECommon _common;
    private string _frsRoot;
    private string _errMsg;

    #region Construction
    // This isn't used but is a best practice to have one in the C# environment.
    public BOERptArchiver()
    {
      throw new Exception("Please do not use the default construction for BOERptArchiver.");
    }

    public BOERptArchiver(string cms, string user, string pw)
    {
      _common = new BOECommon(user, pw, cms);
      _frsRoot = GetOutputServerPath();
    }

    ~BOERptArchiver()
    {
      Dispose();
    }

    public void Dispose()
    {
      _common.Dispose();
      System.GC.SuppressFinalize(this);
    }

    public string ErrMsg
    {
      get { return _errMsg; }
    }
    #endregion Construction

    #region Public

    public List<BOERptInstance> GetArchiveInfo(Int32 rptID, DateTime lastArchive)
    {
      InfoObjects boInstances = GetInstances(rptID, lastArchive);
      List<BOERptInstance> fileInstances = new List<BOERptInstance>();

      try
      {
        try
        {
          string rptFile;
          DateTime rptDate;
          List<BOERptParam> rptParams;
          Report rpt;

          for (int i = 1; i <= boInstances.Count; i++)
          {
            rpt = (Report)boInstances[i];
            rptParams = GetRptParams(rpt);
            rptFile = rpt.Files[1].Name.Replace("frs://Output", _frsRoot);
            rptFile = rptFile.Replace('/', '\\');
            rptDate = Convert.ToDateTime(rpt.Properties["SI_UPDATE_TS"]);
            fileInstances.Add(new BOERptInstance(rptFile, rptDate, rptParams));
          }
        }
        finally
        {
          boInstances.Dispose();
        }
      }
      catch (Exception ex)
      {
        _errMsg = "GetArchiveInfo: " + ex.Message;
      }
      return fileInstances;
    }

    public bool RemoveOldInstances(Int32 rptID, DateTime isOldRptDate)
    {
      bool result = true;
      string compareDate = (isOldRptDate.ToUniversalTime()).ToString("yyyy.MM.dd.hh.mm.ss");
      string query =
        "Select * " +
        "From CI_INFOOBJECTS Where SI_INSTANCE_OBJECT=1 " +
        "And SI_SCHEDULE_STATUS=1 " +
        "And SI_PARENTID=" + rptID +
        " AND SI_UPDATE_TS<='" + compareDate + "'";
      try
      {
        InfoObjects instances = _common.BOEInfoStore.Query(query);
        for (int i = instances.Count; i > 0; i--)
        {
          instances.Delete(instances[i]);
        }
        _common.BOEInfoStore.Commit(instances);
      }
      catch (Exception ex)
      {
        _errMsg = "RemoveOldInstances: " + ex.Message;
        result = false;
      }
      return result;
    }

    #endregion Public

    #region Private
    private InfoObjects GetInstances(int rptID, DateTime lastArchive)
    {
      try
      {
        string compareDate = (lastArchive.ToUniversalTime()).ToString("yyyy.MM.dd.hh.mm.ss");
        string query =
          "Select SI_NAME, SI_ID, SI_DESCRIPTION, SI_UPDATE_TS, SI_FILES, SI_PROMPTS " +
          "From CI_INFOOBJECTS Where SI_INSTANCE_OBJECT=1 " +
          "And SI_SCHEDULE_STATUS=1 " +
          "And SI_PARENTID=" + rptID +
          " AND SI_UPDATE_TS>='" + compareDate + "'";
        InfoObjects instances = _common.BOEInfoStore.Query(query);
        return instances;
      }
      catch (Exception ex)
      {
        throw (new Exception("GetInstances: " + ex.Message));
      }
    }

    private string GetOutputServerPath()
    {
      string result = string.Empty;
      string query = "Select * From CI_SYSTEMOBJECTS Where SI_FRIENDLY_NAME like 'Output%'";
      using (InfoObjects servers = _common.BOEInfoStore.Query(query))
      {
        if (servers.Count >= 1)
        {
          Server server = (Server)servers[1];
          FileServerAdmin serverAdmin = new FileServerAdmin(server.ServerAdmin);
          result = serverAdmin.RootDirectory;
          //strip off trailing backslash
          if (result.Substring(result.Length - 1) == "\\")
            result = result.Substring(1, result.Length - 1);
        }
      }
      return result;
    }

    private List<BOERptParam> GetRptParams(Report rpt)
    {
      try
      {
        List<BOERptParam> paramList = new List<BOERptParam>();
        return paramList;
      }
      catch (Exception ex)
      {
        throw (new Exception("GetRptParams: " + ex.Message));
      }
    }

    #endregion Private
  }
}
