using System;
using System.Collections.Generic;
using System.Text;

namespace ZCSterling.BOE
{

  /// <summary>
  /// Class that holds information about a single report parameter.
  /// </summary>
  public class BOERptParam
  {
    private string _pname;
    private BOEParamType _ptype;
    private string _pvalue;
    private bool _isRange;
    private string _endValue;

    /// <summary>
    /// Default constructor.  Included in case we need to serialize the object.
    /// DO NOT USE!
    /// </summary>
    public BOERptParam()
    {
      //default constructor - this will generally NOT be used!
    }

    public BOERptParam(string ParameterName, BOEParamType ParameterType, string ParameterValue)
    {
      _pname = ParameterName;
      _ptype = ParameterType;
      _pvalue = ParameterValue;
      _isRange = false;
      _endValue = string.Empty;
    }

    public BOERptParam(string ParameterName, BOEParamType ParameterType, string StartValue, string EndValue)
    {
      _pname = ParameterName;
      _ptype = ParameterType;
      _isRange = true;
      _pvalue = StartValue;
      _endValue = EndValue;
    }

    public string ParamName
    {
      get
      {
        return _pname;
      }
    }

    public BOEParamType ParamType
    {
      get
      {
        return _ptype;
      }
    }

    public string ParamValue
    {
      get
      {
        return _pvalue;
      }
      set { _pvalue = value; }
    }

    public bool IsRangeParam
    {
      get { return _isRange; }
    }

    public string EndValue
    {
      get { return _endValue; }
    }
  }

  public enum BOEParamType
  {
    String,
    Number,
    Boolean,
    Date
  }

  /// <summary>
  /// Class that holds information about a single report instance, including its parameters
  /// </summary>
  public class BOERptInstance
  {
    private string _fileName;
    private DateTime _runDate;
    private List<BOERptParam> _params;

    public BOERptInstance()
    {
      _params = new List<BOERptParam>();
    }

    public BOERptInstance(string fileName, DateTime runDate, List<BOERptParam> paramList)
    {
      _fileName = fileName;
      _runDate = runDate;
      _params = paramList;
    }

    public string FileName
    {
      get { return _fileName; }
    }

    public DateTime RunDate
    {
      get { return _runDate; }
    }

    public List<BOERptParam> RptParams
    {
      get { return _params; }
    }
  }


}
